# -*- coding: utf-8 -*-
from plivo.utils.validators import *

from ..base import ListResponseObject, PlivoResource, PlivoResourceInterface
from ..exceptions import *
from ..utils import *

class Campaign(PlivoResource):
    _name = 'Campaign'
    _identifier_string = 'campaign_id'

class Campaign(PlivoResourceInterface):
    _resource_type = Campaign

    @validate_args(campaign_id=[of_type(six.text_type)])
    def get(self, campaign_id):
        return self.client.request(
            'GET', ('10dlc','Campaign', campaign_id), response_type=None)

    @validate_args(campaign_id=[required(of_type(six.text_type))])
    def delete(self, campaign_id):
        return self.client.request(
            'DELETE', ('10dlc','Campaign', campaign_id), response_type=None)
    
    @validate_args(
        campaign_id=[required(of_type(six.text_type))],
        reseller_id=[optional(of_type(six.text_type))],
        description=[optional(of_type(six.text_type))],
        sample1=[optional(of_type(six.text_type))],
        sample2=[optional(of_type(six.text_type))],
        help_message=[optional(of_type(six.text_type))],
        message_flow=[optional(of_type(six.text_type))],
        optin_keywords=[optional(of_type(six.text_type))],
        optin_message=[optional(of_type(six.text_type))],
        optout_keywords=[optional(of_type(six.text_type))],
        optout_message=[optional(of_type(six.text_type))],
        help_keywords=[optional(of_type(six.text_type))],
        )
    def update(self, campaign_id, reseller_id='', description='', sample1='', sample2='', help_message='', message_flow='', optin_keywords='', optin_message='', optout_keywords='', optout_message='', help_keywords=''):
        
        payload = locals()
        payload.pop("campaign_id")
        return self.client.request(
            'POST', ('10dlc','Campaign', campaign_id), to_param_dict(self.update, payload))

    @validate_args(
        brand=[optional(of_type(six.text_type))],
        usecase=[optional(of_type(six.text_type))],
        limit=[optional(of_type(*six.integer_types))],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ])
    def list(self, brand=None, usecase=None, 
                limit=None, offset=None):
        return self.client.request(
            'GET', ('10dlc', 'Campaign', ),
            to_param_dict(self.list, locals()),
            response_type=None,
            objects_type=None)
    
    @validate_args(
        brand_id=[required(of_type(six.text_type))],
        campaign_alias=[optional(of_type(six.text_type))],
        vertical=[required(of_type(six.text_type))],
        usecase=[required(of_type(six.text_type))],
        description=[optional(of_type(six.text_type))],
        embedded_link=[optional(of_type_exact(bool))],
        embedded_phone=[optional(of_type_exact(bool))],
        age_gated=[optional(of_type_exact(bool))],
        direct_lending=[optional(of_type_exact(bool))],
        subscriber_optin=[optional(of_type_exact(bool))],
        subscriber_optout=[optional(of_type_exact(bool))],
        subscriber_help=[optional(of_type_exact(bool))],
        affiliate_marketing=[optional(of_type_exact(bool))],
        sample1=[optional(of_type(six.text_type))],
        sample2=[optional(of_type(six.text_type))],
        sample3=[optional(of_type(six.text_type))],
        sample4=[optional(of_type(six.text_type))],
        sample5=[optional(of_type(six.text_type))],
        message_flow=[optional(of_type(six.text_type))],
        help_message=[optional(of_type(six.text_type))],
        optin_keywords=[optional(of_type(six.text_type))],
        optin_message=[optional(of_type(six.text_type))],
        optout_keywords=[optional(of_type(six.text_type))],
        optout_message=[optional(of_type(six.text_type))],
        help_keywords=[optional(of_type(six.text_type))],
        url=[optional(of_type(six.text_type))],
        method=[optional(of_type(six.text_type))])
    def create(self,
               brand_id,
               vertical,
               usecase,
               description='',
               sample1='',
               sample2='',
               sample3='',
               sample4='',
               sample5='',
               url='',
               message_flow='',
               help_message='',
               optin_keywords='',
               optin_message='',
               optout_keywords='',
               optout_message='',
               help_keywords='',
               method='POST',
               embedded_link=False,
               embedded_phone=False,
               age_gated=False,
               direct_lending=False,
               subscriber_optout=False,
               subscriber_optin=False,
               subscriber_help=False,
               affiliate_marketing=False,
               campaign_alias=None,
               sub_usecases=[]):
        return self.client.request('POST', ('10dlc', 'Campaign'),
                                   to_param_dict(self.create, locals()))


    @validate_args(
        campaign_id=[required(of_type(six.text_type))],
        url=[optional(of_type(six.text_type))],
        method=[optional(of_type(six.text_type))],
    )
    def number_link(self,
                    campaign_id,
                    url='',
                    method='POST',
                    numbers=[]):
        return self.client.request('POST', ('10dlc', 'Campaign',  campaign_id, 'Number'),
                                   to_param_dict(self.number_link, locals()))

    @validate_args(
        campaign_id=[of_type(six.text_type)],
        limit=[optional(of_type(*six.integer_types))],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ])
    def get_numbers(self,
                    campaign_id,
                    limit=None, 
                    offset=None):
        params ={}
        if limit:
            params['limit']= limit
        if offset:
            params['offset'] = offset
        return self.client.request('GET', ('10dlc', 'Campaign',  campaign_id, 'Number'),
                                   params, to_param_dict(self.create, locals()))

    @validate_args(
        campaign_id=[of_type(six.text_type)],
        number=[of_type(six.text_type)])
    def get_number(self,
                campaign_id,
                number):
        return self.client.request('GET', ('10dlc', 'Campaign',  campaign_id, 'Number', number),
                                   to_param_dict(self.create, locals()))

    @validate_args(
        campaign_id=[of_type(six.text_type)],
        number=[of_type(six.text_type)])
    def number_unlink(self,
                campaign_id,
                number,
                url='',
                method='POST'
                ):
        return self.client.request('DELETE', ('10dlc', 'Campaign',  campaign_id, 'Number', number),
                                   to_param_dict(self.create, locals()))


# -*- coding: utf-8 -*-
from plivo.utils import to_param_dict
from plivo.utils.validators import *

from ..base import ListResponseObject, PlivoResource, PlivoResourceInterface


class Endpoint(PlivoResource):
    _name = 'Endpoint'
    _identifier_string = 'endpoint_id'

    @validate_args(
        password=[of_type(six.text_type)],
        alias=[of_type(six.text_type)],
        app_id=[optional(of_type(six.text_type))])
    def update(self, password=None, alias=None, app_id=None):
        params = to_param_dict(self.update, locals())
        self.__dict__.update(params)
        return self.client.endpoints.update(self.id, **params)

    def delete(self):
        return self.client.endpoints.delete(self.id)


class Endpoints(PlivoResourceInterface):
    _resource_type = Endpoint

    @validate_args(
        username=[of_type(six.text_type)],
        password=[of_type(six.text_type)],
        alias=[of_type(six.text_type)],
        app_id=[optional(of_type(six.text_type))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def create(self, username, password, alias, app_id=None, callback_url=None, callback_method=None):
        return self.client.request('POST', ('Endpoint', ),
                                   to_param_dict(self.create, locals()), is_voice_request=True)

    @validate_args(endpoint_id=[of_type(six.text_type)],
                   callback_url=[optional(is_url())],
                   callback_method=[optional(of_type(six.text_type))],
                   )
    def get(self, endpoint_id, callback_url=None, callback_method=None):
        return self.client.request('GET', ('Endpoint', endpoint_id), to_param_dict(self.get, locals()), is_voice_request=True)

    @validate_args(
        limit=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda limit: 0 < limit <= 20, '0 < limit <= 20')))
                ],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def list(self, limit=20, offset=0, callback_url=None, callback_method=None):
        if callback_url:
            return self.client.request(
                'GET',
                ('Endpoint',),
                to_param_dict(self.list, locals()),
                objects_type=Endpoint, is_voice_request=True)
        else:
            return self.client.request(
                'GET',
                ('Endpoint',),
                to_param_dict(self.list, locals()),
                objects_type=Endpoint,
                response_type=ListResponseObject, is_voice_request=True)


    @validate_args(
        endpoint_id=[of_type(six.text_type)],
        password=[optional(of_type(six.text_type))],
        alias=[optional(of_type(six.text_type))],
        app_id=[optional(of_type(six.text_type))],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],
    )
    def update(self, endpoint_id, password=None, alias=None, app_id=None, callback_url=None, callback_method=None):

        # not using locals() because we need to neglect endpoint_id
        temp = {
            'self': self,
            'password': password,
            'alias': alias,
            'app_id': app_id,
            'callback_url': callback_url,
            "callback_method": callback_method
        }
        return self.client.request('POST', ('Endpoint', endpoint_id),
                                   to_param_dict(self.update, temp), is_voice_request=True)

    @validate_args(endpoint_id=[of_type(six.text_type)],
                   callback_url=[optional(is_url())],
                   callback_method=[optional(of_type(six.text_type))],
                   )
    def delete(self, endpoint_id, callback_url=None, callback_method=None):
        return self.client.request('DELETE', ('Endpoint', endpoint_id), to_param_dict(self.delete, locals()),
                                   is_voice_request=True)



from plivo.utils.validators import *
from plivo.xml import PlivoXMLElement, map_type


class PhonemeElement(PlivoXMLElement):
    _name = 'phoneme'
    _nestable = []

    @property
    def alphabet(self):
        return self.__alphabet

    @alphabet.setter
    def alphabet(self, value):
        self.__alphabet = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_alphabet(self, value):
        self.alphabet = value
        return self

    @property
    def ph(self):
        return self.__ph

    @ph.setter
    def ph(self, value):
        self.__ph = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_ph(self, value):
        self.ph = value
        return self

    def __init__(
        self,
        content=None,
        alphabet=None,
        ph=None,
    ):

        super(PhonemeElement, self).__init__()
        self.content = content
        self.alphabet = alphabet
        self.ph = ph

    def to_dict(self):
        d = {
            'alphabet': self.alphabet,
            'ph': self.ph,
        }

        return {
            k: six.text_type(map_type(v))
            for k, v in d.items() if v is not None
        }



# -*- coding: utf-8 -*-
from .accounts import Accounts, Subaccounts
from .applications import Applications
from .calls import Calls
from .token import Token
from .conferences import Conferences
from .endpoints import Endpoints
from .messages import Messages
from .numbers import Numbers
from .phlos import Phlos
from .pricings import Pricings
from .recordings import Recordings
from .addresses import Addresses
from .media import Media
from .identities import Identities
from .call_feedback import CallFeedback
from .powerpacks import Powerpacks
from .lookup import Lookup
from .brand import Brand
from .campaign import Campaign
from .profile import Profile
from .multipartycall import MultiPartyCalls, MultiPartyCall, MultiPartyCallParticipant



from plivo.base import PlivoResource, PlivoResourceInterface

from ..utils.validators import *


class QueuedCall(PlivoResource):
    _name = 'QueuedCall'
    _identifier_string = 'call_uuid'


class QueuedCalls(PlivoResourceInterface):
    _resource_type = QueuedCall
    _iterable = False

    @validate_args(
        limit=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(
                        lambda limit: 0 < limit <= 20,
                        message='0 < limit <= 20')))
        ],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, message='0 <= offset')))
        ],
        callback_url=[optional(is_url())],
        callback_method=[optional(of_type(six.text_type))],)
    def list_ids(self, limit=20, offset=0, callback_url=None, callback_method=None):
        return self.client.request('GET', ('Call', ), {
            'status': 'queued',
            'limit': limit,
            'offset': offset,
            'callback_url': callback_url,
            'callback_method': callback_method
        }, is_voice_request=True)

    @validate_args(_id=[of_type(six.text_type)],
                   callback_url=[optional(is_url())],
                   callback_method=[optional(of_type(six.text_type))],)
    def get(self,
            _id,
            callback_url=None,
            callback_method=None):
        local_object={}
        local_object['status'] = 'queued'
        if callback_url:
            local_object['callback_url'] = callback_url
        if callback_method:
            local_object['callback_method'] = callback_method

        return self.client.request('GET', ('Call', _id), local_object, is_voice_request=True)
